package com.theitbulls.downloadfile;

import android.app.Activity;
import android.app.IntentService;
import android.content.Intent;
import android.os.Environment;
import android.util.Log;

import androidx.annotation.Nullable;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

public class DownloadFileIntentService extends IntentService {
    public static final String DOWNLOAD_INTENT = "com.theitbulls.downloadfile";

    public static final String URL = "fileUrl";
    public static final String FILE_NAME = "filename";

    public static final String RESULT = "result";
    public static final String RESULT_MSG = "msg";

    public DownloadFileIntentService() {
        super("File Downloading Intent");
    }

    @Override
    protected void onHandleIntent(@Nullable Intent intent) {
        if (intent == null) {
            return;
        }

        if (DOWNLOAD_INTENT.equals(intent.getAction())) {
            String fileUrl = intent.getStringExtra(URL);
            String fileName = intent.getStringExtra(FILE_NAME);

            File output = new File(Environment.getExternalStorageDirectory(), fileName);
            output.deleteOnExit();

            InputStream io = null;
            FileOutputStream fos = null;

            try {
                URL url = new URL(fileUrl);
                io = url.openConnection().getInputStream();

                byte[] fileData = readFileData(io);
                fos = new FileOutputStream(output.getPath());
                fos.write(fileData);
                fos.flush();

                sendResult(output.getPath(), "Success", Activity.RESULT_OK);
            } catch (Exception e) {
                Log.e("DownloadError: ", e.toString());
                sendResult(output.getPath(), e.toString(), Activity.RESULT_CANCELED);
            } finally {
                close(io, fos);
            }
        }
    }

    private void sendResult(String url, String msg, int resultCode) {
        Intent resultIntent = new Intent(DOWNLOAD_INTENT);
        resultIntent.putExtra(URL, url);
        resultIntent.putExtra(RESULT, resultCode);
        resultIntent.putExtra(RESULT_MSG, msg);
        sendBroadcast(resultIntent);
    }

    private byte[] readFileData(InputStream io) throws IOException {
        ByteArrayOutputStream ous = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int read = 0;
        while ((read = io.read(buffer)) != -1) {
            ous.write(buffer, 0, read);
        }
        return ous.toByteArray();
    }

    private void close(InputStream io, FileOutputStream fos) {
        try {
            io.close();
        } catch (Exception e) {
        }
        try {
            fos.close();
        } catch (Exception e) {
        }

    }
}
